#!/usr/bin/env -S bash

set -o errexit
set -o nounset
set -o pipefail

secretsEncryptString="Secrets encryption configuration found: true"

check_encryption_provider() {
    output=$(kubectl cluster-info dump)
    # Grabbing the file locations of the existing encryption providers (if any) for parsing
    config_files=$(echo "$output" | grep -oP 'encryption-provider-config\s*=\s*\K[^,"]+\\?' || true)
    config_files=$(echo "$config_files" | sed 's/\\$//')
    if [[ -n "$config_files" ]]; then
        for config_file in $config_files; do
            if [[ -f "$config_file" ]]; then
                provider_output=$(/usr/bin/kpec-cli check-provider "$config_file" 2>&1)
                # Parse the output for 'Secrets encryption configuration found' 
                if echo "$provider_output" | grep -q "$secretsEncryptString"; then
                    echo "$provider_output"
                    # Further parse output to check for KPEC
                    if echo "$provider_output" | grep -q "kpec-provider"; then
                        echo "K3s is already configured with KPEC as the encryption provider for secrets. Please run apt-get upgrade kpec if you would like to update to the latest version."
                        exit 1
                    else
                        echo "Error: K3s is already configured with an encryption provider for secrets. Please first uninstall the existing encryption provider before re-attempting installation of KMS Plugin for Edge Clusters."
                        exit 1
                    fi
                fi
            else
                handle_cannot_find_config_file "$config_file"
            fi
        done
    fi
}

handle_cannot_find_config_file() {
    echo "Error: We have detected one or more existing encryption providers, but cannot verify whether or not they are configured to encrypt k8s secrets."
    echo "Installation of the KMS Plugin for Edge Clusters while the cluster is configured with another encryption provider for k8s secrets could cause the cluster to become unrecoverable."
    echo
    echo "If the existing encryption provider is configured to encrypt k8s secrets, please first uninstall this encryption provider and then re-run this script."
    echo "If the existing encryption provider IS NOT configured to encrypt k8s secrets, you may proceed with this installation."
    echo
    read -p "Proceed with this installation? (y/n): " answer
    answer=$(echo "$answer" | xargs)
    if [[ $answer =~ ^[Yy]$ ]]; then
        echo "Proceeding with installation..."
    else
        echo "Installation aborted."
        exit 1
    fi
}

check_k3s_secrets_encryption() {
    status=$(k3s secrets-encrypt status)
    if echo "$status" | grep 'Encryption Status: Enabled' > /dev/null; then
        echo "Error: K3s secrets encryption is already enabled."
        echo "Please disable the existing k3s secrets encryption before proceeding."
        exit 1
    fi
}

check_encryption_provider

check_k3s_secrets_encryption