#!/usr/bin/env bash

set -o errexit
set -o nounset
set -o pipefail

/usr/share/kpec/kpec_utilities.sh check_if_root

/usr/share/kpec/kpec_utilities.sh wait_for_k3s

/usr/share/kpec/kpec_utilities.sh wait_for_kpec

/usr/share/kpec/kpec_utilities.sh rotate_secrets

# Change principal encryption provider to identity
rm -f /etc/rancher/k3s/encryption-config.yaml
cp -v /usr/share/kpec/revert-config.yaml /etc/rancher/k3s/
chown kpec /etc/rancher/k3s/revert-config.yaml
chmod u=rw,g=rw,o= /etc/rancher/k3s/revert-config.yaml

rm -f /etc/rancher/k3s/config.yaml.d/k3s-kms-config.yaml
cp -v /usr/share/kpec/k3s-kms-revert-config.yaml /etc/rancher/k3s/config.yaml.d/
chown kpec /etc/rancher/k3s/config.yaml.d/k3s-kms-revert-config.yaml
chmod u=rw,g=rw,o= /etc/rancher/k3s/config.yaml.d/k3s-kms-revert-config.yaml

# Restart k3s
read -p "K3s must be restarted to reset encryption provider to default, restart now? (y/n): " answer
answer=$(echo "$answer" | xargs)
if [[ $answer =~ ^[Yy]$ ]]; then
    echo "Proceeding with K3s restart..."
    systemctl restart k3s

    /usr/share/kpec/kpec_utilities.sh wait_for_k3s
else
    echo "Skipping K3s restart, please restart manually with 'sudo systemctl restart k3s'"
    exit 0
fi

# sleep to allow object modification
sleep 20

# Replace secrets using identity encryption provider
/usr/share/kpec/kpec_utilities.sh rotate_secrets

# Remove all config files and kpec-keks
rm -f /etc/rancher/k3s/revert-config.yaml
rm -f /etc/rancher/k3s/config.yaml.d/k3s-kms-revert-config.yaml
rm -f /etc/rancher/k3s/encryption-config.yaml.bak

rm -f /etc/cron.d/kpec_rotate_keks

# Restart k3s
read -p "K3s must be restarted again to fully configure changes, restart now? (y/n): " answer
answer=$(echo "$answer" | xargs)
if [[ $answer =~ ^[Yy]$ ]]; then
    echo "Proceeding with K3s restart..."
    systemctl restart k3s

    /usr/share/kpec/kpec_utilities.sh wait_for_k3s
else
    echo "Skipping K3s restart, please restart manually with 'sudo systemctl restart k3s'"
    exit 0
fi

exit 0
