#!/usr/bin/env -S bash

set -o errexit
set -o nounset
set -o pipefail

/usr/share/kpec/kpec_utilities.sh check_if_root

/usr/share/kpec/kpec_utilities.sh wait_for_k3s

# Different distributions policies may prevent kpec from being started on install,
# this will start kpec if it has not already been started.
sudo systemctl start kpec

/usr/share/kpec/kpec_utilities.sh wait_for_kpec

if kubectl cluster-info dump | grep encryption-provider; then
    echo "Error: K3s is already configured with an encryption-provider, please first uninstall the existing encryption-provider and then re-attempt install of KMS Plugin for Edge Clusters."
    exit 1
fi

# KPEC must be installed on a control-plane node
/usr/share/kpec/kpec_utilities.sh verify_control_plane_node

sudo /usr/share/kpec/check_for_kms_provider.sh

cp -v /usr/share/kpec/encryption-config.yaml /etc/rancher/k3s/
chown kpec /etc/rancher/k3s/encryption-config.yaml
chmod u=rw,g=rw,o= /etc/rancher/k3s/encryption-config.yaml

mkdir -p /etc/rancher/k3s/config.yaml.d
cp -v /usr/share/kpec/k3s-kms-config.yaml /etc/rancher/k3s/config.yaml.d/
chown kpec /etc/rancher/k3s/config.yaml.d/k3s-kms-config.yaml
chmod u=rw,g=rw,o= /etc/rancher/k3s/config.yaml.d/k3s-kms-config.yaml

# Restart K3s to apply services
read -p "K3s must be restarted for KMS Plugin for Edge Clusters installation to take effect, restart now? (y/n): " answer
answer=$(echo "$answer" | xargs)
if [[ $answer =~ ^[Yy]$ ]]; then
    echo "Proceeding with K3s restart..."
    systemctl restart k3s

    /usr/share/kpec/kpec_utilities.sh wait_for_k3s

    /usr/share/kpec/kpec_utilities.sh wait_for_kpec
else
    echo "Skipping K3s restart, please restart manually with 'sudo systemctl restart k3s'"
    exit 0
fi

# sleep to allow object modification
sleep 20

/usr/bin/kpec-cli rotate-kek

/usr/share/kpec/kpec_utilities.sh rotate_secrets

touch /usr/share/kpec/kek-rotation-timestamp
touch /etc/cron.d/kpec_rotate_keks

# cron job executes every hour to check KEK age and rotate if KEK is older than 30 days
echo '0 * * * * root /usr/share/kpec/check_kek_age_and_rotate.sh 30 >> /var/log/kpec/kpec-cli.log 2>&1' > /etc/cron.d/kpec_rotate_keks

echo "KMS Plugin for Edge Clusters has been deployed successfully."
echo "The etcdctl utility can be used to verify that secrets are encrypted in the etcd store, for more information visit https://etcd.io/docs/v3.4/dev-guide/interacting_v3/"
echo "For more information on the KMS Plugin for Edge Clusters, please visit https://aepreviews.ms/docs/kms-plugin-for-edge-clusters/."

exit 0
